use chrono::{Local, Timelike};
use clap::{Parser, Subcommand};
use std::{error::Error, path::PathBuf};
use sunrise::Sunrise;

mod sunrise;

#[derive(Parser)]
#[command(version, about, long_about = None)]
struct Cli {
    #[command(subcommand)]
    command: Commands,
}

#[derive(Subcommand)]
enum Commands {
    /// Print the current wallpaper
    Current {
        /// Sunrise file
        file: PathBuf,
    },

    /// Print the time to apply the next wallpaper
    NextAt {
        /// Sunrise file
        file: PathBuf,
    },

    /// Print the time til the next wallpaper
    NextIn {
        /// Sunrise file
        file: PathBuf,

        /// Display time in seconds
        #[arg(short,long)]
        seconds: bool,
    },
}

fn sec_to_readable(sec: u32) -> String {
    let hours = sec / 3600;
    let minutes = (sec % 3600) / 60;
    let seconds = sec % 60;

    format!("{:02}:{:02}:{:02}", hours, minutes, seconds)
}

fn main() -> Result<(), Box<dyn Error>> {
    let cli = Cli::parse();

    match cli.command {
        Commands::Current { file } => {
            let sunrise = Sunrise::load_from_file(&file)?;
            let now = Local::now();
            let sec = now.hour() * 3600 + now.minute() * 60 + now.second();
            match sunrise.closest_passed_time(sec) {
                Some(timetable) => match file.canonicalize()?.parent() {
                    Some(sunrise_absolute_base) => {
                        let image = timetable.get_absolute_image_path(&sunrise_absolute_base);
                        println!("{}", image.to_str().unwrap());
                    }
                    None => {
                        println!("Failed");
                    }
                },

                None => {
                    println!("Failed");
                }
            }
        }

        Commands::NextAt { file } => {
            let sunrise = Sunrise::load_from_file(&file)?;
            let now = Local::now();
            let sec = now.hour() * 3600 + now.minute() * 60 + now.second();

            match sunrise.next_time(sec) {
                Some(timetable) => {
                    println!("{}", sec_to_readable(timetable.time));
                }
                None => {
                    println!("Failed");
                }
            }
        }

        Commands::NextIn { file, seconds } => {
            let sunrise = Sunrise::load_from_file(&file)?;
            let now = Local::now();
            let sec = now.hour() * 3600 + now.minute() * 60 + now.second();

            match sunrise.next_time(sec) {
                Some(timetable) => {
                    let next_in = if timetable.time >= sec {
                        timetable.time - sec
                    }else{
                        timetable.time + (24 * 60 * 60) - sec
                    };

                    if seconds {
                        println!("{}",next_in);
                    }else{
                        println!("{}",sec_to_readable(next_in));
                    }
                    
                }
                None => {
                    println!("Failed");
                }
            }
        }
    }

    Ok(())
}
